@@ -78,23 +78,28 @@ class TracebackData:
         self.bad_line = "\n"
         self.filename = ""
         self.exception_frame = None
+        self.program_stopped_frame = None
+        self.program_stopped_bad_line = "\n"
         self.get_source_info()
 
-        # The following four attributes get their correct values in locate_error()
+        # The following attributes get their correct values in self.locate_error()
         self.node = None
         self.node_text = ""
         self.node_range = None
-        self.original_bad_line = ""
+        self.original_bad_line = self.bad_line
+        self.program_stopped_node_range = None
 
         if issubclass(etype, SyntaxError):
             self.statement = source_info.Statement(self.value, self.bad_line)
             # Removing extra ending spaces for potentially shorter displays later on
+
             def remove_space(text):
                 if text.rstrip():
                     if text.endswith("\n"):
                         return text.rstrip() + "\n"
                     return text.rstrip()
                 return text
+
             self.statement.statement = remove_space(self.statement.statement)
             self.statement.bad_line = remove_space(self.statement.bad_line)
         else:
@@ -134,10 +139,9 @@ class TracebackData:
             # so that we can use these elsewhere without having to perform
             # additional checks.
             if not hasattr(self.value, "end_offset"):
-                if self.value.offset:
-                    self.value.end_offset = self.value.offset + 1
-                else:
-                    self.value.end_offset = 0
+                self.value.end_offset = (
+                    self.value.offset + 1 if self.value.offset else 0
+                )
                 self.value.end_lineno = self.value.lineno
             if self.value.text is not None:
                 self.bad_line = self.value.text  # typically includes "\n"
@@ -154,29 +158,94 @@ class TracebackData:
 
         if self.records:
             self.exception_frame, self.filename, linenumber, _, _, _ = self.records[-1]
-            _, line = cache.get_formatted_partial_source(
-                self.filename, linenumber, None
-            )
+            _, line = cache.get_formatted_partial_source(self.filename, linenumber)
             self.bad_line = line.rstrip()
+            if len(self.records) > 1:
+                self.program_stopped_frame, filename, linenumber, *_rest = self.records[
+                    0
+                ]
+                _, line = cache.get_formatted_partial_source(filename, linenumber)
+                self.program_stopped_bad_line = line.rstrip()
             return
 
         # We should never reach this stage.
-        if True:  # pragma: no cover
+        def _log_error():  # pragma: no cover
             debug_helper.log("Internal error in TracebackData.get_source_info.")
             debug_helper.log("No records found.")
             debug_helper.log("self.exception_type:" + str(self.exception_type))
             debug_helper.log("self.value:" + str(self.value))
             debug_helper.log_error()
 
+        _log_error()  # pragma: no cover
+
     def locate_error(self, tb):
-        """Attempts to narrow down the location of the error."""
+        """Attempts to narrow down the location of the error so that,
+        if possible, the problem code is highlighted with ^^^^."""
         if not self.records:  # pragma: no cover
             debug_helper.log("No records in locate_error().")
             return
 
-        # 1) If we can find the precise location (node) on a line of code
-        #    causing the exception, we note this location
-        #    so that we can indicate it later with ^^^^^, something like:
+        if self.program_stopped_frame is not None:
+            exc_tb = self.find_tb_frame(tb, self.program_stopped_frame)
+            if exc_tb is not None:
+                _, self.program_stopped_node_range, _ = self.find_node(
+                    exc_tb, self.program_stopped_bad_line
+                )
+        if self.exception_name == "NameError":
+            # `executing` cannot give us the node location in this case
+            return self.locate_name_error()
+
+        tb = self.find_tb_frame(tb, self.exception_frame)
+        if tb is None:
+            debug_helper.log("Exception frame could not be found.")  # pragma: no cover
+            return  # pragma: no cover
+
+        self.node, self.node_range, self.node_text = self.find_node(tb, self.bad_line)
+        if self.node_text.strip():
+            # Replacing the line that caused the exception by the text
+            # of the 'node' facilitates the process of identifying the cause.
+            # However, in a few cases, we do need to keep the entire original line.
+            self.original_bad_line = self.bad_line
+            self.bad_line = self.node_text
+
+    @staticmethod
+    def find_tb_frame(tb, frame):
+        """Find a traceback object where a given frame resides."""
+        while True:
+            if tb.tb_frame == frame:
+                return tb
+            tb = tb.tb_next
+            if not tb:  # pragma: no cover
+                debug_helper.log("No tb_frame found.")
+                return None
+
+    def locate_name_error(self):
+        """Finds the location of an unknown name"""
+        name, _ignore = name_error.get_unknown_name(self.message)
+
+        if name is not None and name in self.bad_line:
+            begin = self.bad_line.find(name)
+            end = begin + len(name)
+            self.node_range = begin, end
+        else:  # pragma: no cover
+            debug_helper.log("Could not locate unknown name.")
+
+    @staticmethod
+    def find_node(tb, bad_line):
+        """Finds the 'node', that is the exact part of a line of code
+        that is related to the cause of the problem.
+        """
+        try:
+            ex = executing.Source.executing(tb)
+            node = ex.node
+            node_text = ex.text()
+        except Exception as e:  # pragma: no cover
+            debug_helper.log("Exception raised in TracebackData.use_executing.")
+            debug_helper.log(str(e))
+            return
+        # If we can find the precise location (node) on a line of code
+        # causing the exception, we note this location
+        # so that we can indicate it later with ^^^^^, something like:
         #
         #    20:     b = tuple(range(50))
         #    21:     try:
@@ -187,65 +256,33 @@ class TracebackData:
         # If the node spans the entire line, we do not bother to indicate
         # its specific location.
         #
-        # 2) Sometimes, a node will span multiple lines. For example,
-        #    line 22 shown above might have been written as:
+        # Sometimes, a node will span multiple lines. For example,
+        # line 22 shown above might have been written as:
         #
         #    print(a[
         #            50], b[0])
         #
         # If that is the case, we rewrite the node as a single line.
-        #
-        # Other than for NameError, we replace our definition (text) of
-        # the line that caused the exception by that of the node itself,
-        # to be used later for processing.
-        if self.exception_name == "NameError":
-            # `executing` cannot give us the node location in this case
-            return self.locate_name_error()
 
-        while True:
-            if tb.tb_frame == self.exception_frame:
-                break
-            tb = tb.tb_next
-            if not tb:  # pragma: no cover
-                debug_helper.log("No tb in locate_error().")
-                return
-
-        try:
-            ex = executing.Source.executing(tb)
-            self.node = ex.node
-            self.node_text = ex.text()
-            # We want to transform logical line (or parts thereof) into
-            # something that fits on a single physical line.
-            # \n could be a valid newline token or a character within
-            # a string; we only want to replace newline tokens.
-            if "\n" in self.node_text:
-                tokens = token_utils.tokenize(self.node_text)
-                tokens = [tok for tok in tokens if tok != "\n"]
-                self.node_text = "".join(tok.string for tok in tokens)
-            _bad_line = token_utils.strip_comment(self.bad_line)
-            if (
-                self.node_text
-                and self.node_text in self.bad_line
-                and self.node_text.strip() != _bad_line.strip()
-            ):
-                begin = self.bad_line.find(self.node_text)
-                end = begin + len(self.node_text)
-                self.node_range = begin, end
-            if self.node_text.strip():
-                self.original_bad_line = self.bad_line
-                self.bad_line = self.node_text
-        except Exception as e:  # pragma: no cover
-            debug_helper.log("Exception raised in TracebackData.use_executing.")
-            debug_helper.log(str(e))
-
-    def locate_name_error(self):
-        """Finds the location of an unknown name"""
-        name, _ignore = name_error.get_unknown_name(self.message)
-
-        if name is not None and name in self.bad_line:
-            begin = self.bad_line.find(name)
-            end = begin + len(name)
-            self.node_range = begin, end
+        # To start, we transform logical line (or parts thereof) into
+        # something that fits on a single physical line.
+        # \n could be a valid newline token or a character within
+        # a string; we only want to replace newline tokens.
+        node_range = None
+        if "\n" in node_text:
+            tokens = token_utils.tokenize(node_text)
+            tokens = [tok for tok in tokens if tok != "\n"]
+            node_text = "".join(tok.string for tok in tokens)
+        bad_code = token_utils.strip_comment(bad_line)
+        if (
+            node_text
+            and node_text in bad_line
+            and node_text.strip() != bad_code.strip()
+        ):
+            begin = bad_line.find(node_text)
+            end = begin + len(node_text)
+            node_range = begin, end
+        return node, node_range, node_text
 
 
 # ====================
@@ -338,11 +375,14 @@ class FriendlyTraceback:
             raise SystemExit
         self.suppressed = ["       ... " + _("More lines not shown.") + " ..."]
         self.info = {"header": _("Python exception:")}
-        self.info["_exc_type"] = etype
-        self.info["_exc_instance"] = value
         self.message = self.assign_message()  # language independent
         self.assign_tracebacks()
 
+        # include some values for debugging purpose in an interactive session
+        self.info["_exc_instance"] = value
+        self.info["_frame"] = self.tb_data.exception_frame
+        self.info["_tb_data"] = self.tb_data
+
     def assign_message(self):
         """Assigns the error message, as the attribute ``message``
         which is something like::
@@ -448,6 +488,10 @@ class FriendlyTraceback:
             self.info["cause"] = cannot_analyze_stdin()
             return
 
+        if "encoding problem" in str(self.tb_data.value):
+            self.info["cause"] = _("The encoding of the file was not valid.\n")
+            return
+
         if etype.__name__ == "IndentationError":
             self.info["cause"] = indentation_error.set_cause_indentation_error(
                 value, self.tb_data.statement
@@ -562,9 +606,11 @@ class FriendlyTraceback:
         if lines == ["\n"] and source_cache.idle_get_lines is not None:
             # skipcq: PYL-E1102
             lines = source_cache.idle_get_lines(filename, linenumber - 1)
-        partial_source = get_partial_source(filename, linenumber, lines, index)
+        partial_source = get_partial_source(
+            filename, linenumber, lines, index, self.tb_data.program_stopped_node_range
+        )
         filename = path_utils.shorten_path(filename)
-        if session.use_rich:
+        if session.use_rich:  # pragma: no cover
             filename = f"`'{filename}'`"
 
         self.info["last_call_header"] = _(
@@ -602,17 +648,9 @@ class FriendlyTraceback:
                 "'{filename}'\n"
                 "beyond the location indicated by ^.\n"
             ).format(filename=path_utils.shorten_path(filepath))
-        elif "unexpected EOF while parsing" in repr(value):
-            self.info["parsing_error"] = _(
-                "Python could not understand the code the file\n"
-                "'{filename}'.\n"
-                "It reached the end of the file and expected more content.\n"
-            ).format(filename=path_utils.shorten_path(filepath))
         elif filepath:  # could be None
             self.info["parsing_error"] = _(
-                "Python could not understand the code in the file\n"
-                "'{filename}'\n"
-                "for an unspecified reason.\n"
+                "Python could not understand the code in the file\n" "'{filename}'.\n"
             ).format(filename=path_utils.shorten_path(filepath))
 
         self.info["parsing_error_source"] = f"{partial_source}\n"
@@ -647,7 +685,11 @@ class FriendlyTraceback:
         shortened_tb = self.shorten(tb)
 
         header = "Traceback (most recent call last):"  # not included in records
-        if python_tb[0].startswith(header):
+        if (
+            python_tb[0].startswith(header)
+            and self.tb_data.filename is not None
+            and "<SyntaxError>" not in self.tb_data.filename  # Latest IDLE hack
+        ):
             tb.insert(0, header)
             shortened_tb.insert(0, header)
 
@@ -785,8 +827,9 @@ class FriendlyTraceback:
                     # However, we ensured prior to reaching this point that
                     # they would be defined for other Python versions
                     if (
-                        value.end_lineno != value.lineno
-                        or value.end_offset
+                        value.end_lineno is not None
+                        and value.end_lineno != value.lineno
+                        or value.end_offset is not None
                         and value.end_offset < 1
                     ):
                         nb_carets = len(bad_line) - offset + 1
@@ -813,12 +856,10 @@ def get_partial_source(filename, linenumber, lines, index, text_range=None):
     )
     if filename in cache.cache:
         source, line = cache.get_formatted_partial_source(
-            filename, linenumber, offset=None, text_range=text_range
+            filename, linenumber, text_range=text_range
         )
     elif filename and os.path.abspath(filename):
-        source, line = highlight_source(
-            linenumber, index, lines, offset=None, text_range=text_range
-        )
+        source, line = highlight_source(linenumber, index, lines, text_range=text_range)
         if not source:  # pragma: no cover
             line = ""
             if filename == "<stdin>":
