@@ -8,7 +8,6 @@ import builtins
 import sys
 
 from . import utils
-from . import debug_helper
 from . import token_utils
 
 from .path_info import path_utils
@@ -18,7 +17,7 @@ from .my_gettext import current_lang
 try:
     from asttokens import ASTTokens  # noqa
     from pure_eval import Evaluator, group_expressions  # noqa
-except ImportError:
+except ImportError:  # pragma: no cover
     pass  # ignore errors when processed by Sphinx
 
 
@@ -96,16 +95,6 @@ def get_all_objects(line, frame):
                     objects["builtins"].append((name, repr(obj), obj))
                     objects["name, obj"].append((name, obj))
 
-    dotted_names = get_dotted_names(line)
-    for name in dotted_names:
-        for scope, scope_dict in scopes:
-            if name not in scope_dict:
-                continue
-            obj = scope_dict[name]
-            if (name, obj) not in objects["name, obj"]:
-                objects[scope].append((name, repr(obj), obj))
-                objects["name, obj"].append((name, obj))
-
     try:
         atok = ASTTokens(line, parse=True)
     except SyntaxError:  # this should not happen
@@ -130,39 +119,6 @@ def get_all_objects(line, frame):
     return objects
 
 
-def get_dotted_names(line):
-    """Retrieve dotted names, i.e. something like A.x or A.x.y, etc.
-
-    In principle, pure_eval/ASTTokens used above should be able to
-    retrieve dotted names. However, I have not (yet) been able to do so
-    without this hack.
-    """
-    names = []
-    prev_token = token_utils.tokenize("")[0]  # convenient guard
-    dot_found = False
-    tokens = token_utils.get_significant_tokens(line)
-    for tok in tokens:
-        if tok == ".":
-            dot_found = True
-            continue
-        if tok.is_identifier():
-            if prev_token.is_identifier() and dot_found:
-                previous_name = names[-1]
-                names.append(previous_name + "." + tok.string)
-                dot_found = False
-                continue
-            names.append(tok.string)
-        prev_token = tok
-        dot_found = False
-
-    # remove duplicate and names without "."
-    dotted_names = []
-    for name in names:
-        if name not in dotted_names and "." in name:
-            dotted_names.append(name)
-    return dotted_names
-
-
 def get_object_from_name(name, frame):
     """Given the name of an object, for example 'str', or 'A' for
     class A, returns a basic object of that type found in a frame,
@@ -222,11 +178,6 @@ def get_variables_in_frame_by_scope(frame, scope):
                 non_locals[key] = frame.f_locals[key]
         return non_locals
 
-    debug_helper.log("Internal error in get_variable_in_frame_by_scope()")
-    debug_helper.log(f"unknown scope '{scope}'")
-    debug_helper.log_error()
-    return {}
-
 
 def get_definition_scope(variable_name, frame):
     """Returns a list of scopes ('local', 'global', 'nonlocal')
@@ -247,7 +198,6 @@ def get_var_info(line, frame):
 
     We ignore values found *only* in nonlocal scope as they should not
     be relevant.
-
     """
 
     names_info = []
@@ -377,15 +327,22 @@ def get_similar_names(name, frame):
     Python's builtins.
     """
     similar = {}
-    locals_ = get_variables_in_frame_by_scope(frame, "local")
-    similar["locals"] = utils.get_similar_words(name, locals_)
-
-    globals_ = get_variables_in_frame_by_scope(frame, "global")
-    names = utils.get_similar_words(name, globals_)
-    similar["globals"] = [name for name in names if name not in similar["locals"]]
-
-    similar["builtins"] = utils.get_similar_words(name, dir(builtins))
-    all_similar = similar["locals"] + similar["globals"] + similar["builtins"]
+    # We need to first combine the candidates from all possible sources
+    # so as to treat them on an equal footing.
+    locals_ = list(frame.f_locals.keys())
+    globals_ = list(frame.f_globals.keys())
+    builtins_ = dir(builtins)
+    all_similar = utils.get_similar_words(name, locals_ + globals_ + builtins_)
+    similar["locals"] = []
+    similar["globals"] = []
+    similar["builtins"] = []
+    for word in all_similar:
+        if word in locals_:
+            similar["locals"].append(word)
+        elif word in globals_:
+            similar["globals"].append(word)
+        else:
+            similar["builtins"].append(word)
     if all_similar:
         most_similar = utils.get_similar_words(name, all_similar)
         similar["best"] = most_similar[0]
