@@ -11,7 +11,7 @@ import colorama
 
 from friendly import install, exclude_file_from_traceback, explain_traceback, session
 from friendly.console_helpers import *  # noqa
-from friendly.console_helpers import helpers  # noqa
+from friendly.console_helpers import __all__  # noqa
 
 try:
     from IPython.utils import py3compat  # noqa
@@ -24,8 +24,6 @@ except Exception:  # noqa
 colorama.deinit()
 colorama.init(convert=False, strip=False)
 
-__all__ = list(helpers.keys())
-
 
 def set_width(width=80):
     """Sets the width in a iPython/Jupyter session using Rich."""
@@ -36,6 +34,8 @@ def set_width(width=80):
 
 
 __all__.append("set_width")
+Friendly.set_width = set_width  # noqa
+
 
 shell.InteractiveShell.showtraceback = lambda self, *args, **kwargs: explain_traceback()
 shell.InteractiveShell.showsyntaxerror = (
