@@ -93,12 +93,3 @@ def no_information():
 def internal_error():
     _ = current_lang.translate
     return _("Internal error for Friendly.\n") + please_report()
-
-
-def use_www():
-    _ = current_lang.translate
-    return _(
-        "If you are using a Friendly console, you might want to\n"
-        "use the function `www()` which will open a browser at\n"
-        "a relevant place in the Python documentation.\n"
-    )
