@@ -12,7 +12,7 @@ from .. import debug_helper
 def get_cause(value, _frame, tb_data):
     try:
         return _get_cause(value, tb_data)
-    except Exception as e:
+    except Exception as e:  # pragma: no cover
         debug_helper.log_error(e)
         return {"cause": internal_error()}
 
