@@ -7,9 +7,15 @@ from .. import debug_helper
 from .. import token_utils
 
 
-# The following is intended to be used in custom environments.
-# Currently, it is only used with running a program in Mu.
+def using_python():  # pragma: no cover
+    _ = current_lang.translate
+    return _("You are already using Python!")
+
+
+# The following is also intended to be used in custom environments;
+# we currently use it in Mu.
 CUSTOM_NAMES = {}
+CUSTOM_NAMES["python"] = using_python
 
 
 def get_cause(value, frame, tb_data):
@@ -54,19 +60,19 @@ def free_variable_referenced(unknown_name, *_args):
     _ = current_lang.translate
     cause = _(
         "In your program, `{var_name}` is an unknown name\n"
-        "that had been declared as belonging in a nonlocal scope;\n"
-        "however, no such name could not be found.\n"
+        "that exists in an enclosing scope,\n"
+        "but has not yet been assigned a value.\n"
     ).format(var_name=unknown_name)
     return {"cause": cause}
 
 
 def name_not_defined(unknown_name, frame, tb_data):
     _ = current_lang.translate
-    cause = _("In your program, `{var_name}` is an unknown name.\n").format(
+    cause = _("In your program, no object with the name `{var_name}` exists.\n").format(
         var_name=unknown_name
     )
 
-    if unknown_name == "ꓺ":
+    if unknown_name == "ꓺ":  # pragma: no cover
         return flipfloperator()
 
     known = is_stdlib_module(unknown_name, tb_data)
@@ -74,8 +80,10 @@ def name_not_defined(unknown_name, frame, tb_data):
         return known
 
     if unknown_name in CUSTOM_NAMES:
-        cause = CUSTOM_NAMES[unknown_name]()
-        return {"cause": cause, "suggest": cause}
+        bad_line = tb_data.bad_line.replace("(", "").replace(")", "").strip()
+        if bad_line == unknown_name:
+            cause = CUSTOM_NAMES[unknown_name]()
+            return {"cause": cause, "suggest": cause}
 
     if unknown_name in ["i", "j"]:
         hint = _("Did you mean `1j`?\n")
@@ -110,7 +118,7 @@ def name_not_defined(unknown_name, frame, tb_data):
     return cause
 
 
-def flipfloperator():
+def flipfloperator():  # pragma: no cover
     _ = current_lang.translate
     hint = _("You must be a fan of PyConAu!\n")
     cause = _(
