@@ -4,40 +4,16 @@ Collection of functions useful in parsing ValueError messages and
 providing a more detailed explanation.
 """
 
+import inspect
 import re
 
-from ..my_gettext import current_lang, no_information, internal_error
+from ..my_gettext import current_lang
 from .. import info_variables
-from .. import debug_helper
+from .. import token_utils
 from .. import utils
 
 convert_type = info_variables.convert_type
-MESSAGES_PARSERS = []
-
-
-def add_message_parser(func):
-    """A simple decorator that adds a function to parse a specific message
-    to the list of known parsers."""
-    MESSAGES_PARSERS.append(func)
-
-
-def get_cause(value, frame, tb_data):
-    try:
-        return _get_cause(value, frame, tb_data)
-    except Exception as e:  # pragma: no cover
-        debug_helper.log_error(e)
-        return {"cause": internal_error(), "suggest": internal_error()}
-
-
-def _get_cause(value, frame, tb_data):
-    _ = current_lang.translate
-
-    message = str(value)
-    for parser in MESSAGES_PARSERS:
-        cause = parser(message, frame, tb_data)
-        if cause:
-            return cause
-    return {"cause": no_information()}
+parser = utils.RuntimeMessageParser()
 
 
 def _unpacking():
@@ -73,7 +49,7 @@ def get_iterable(code, frame):
     return obj, iterable
 
 
-@add_message_parser
+@parser.add
 def not_enough_values_to_unpack(message, frame, tb_data):
     _ = current_lang.translate
     pattern1 = re.compile(r"not enough values to unpack \(expected (\d+), got (\d+)\)")
@@ -113,7 +89,7 @@ def not_enough_values_to_unpack(message, frame, tb_data):
     return {"cause": cause}
 
 
-@add_message_parser
+@parser.add
 def too_many_values_to_unpack(message, frame, tb_data):
     _ = current_lang.translate
     pattern = re.compile(r"too many values to unpack \(expected (\d+)\)")
@@ -150,7 +126,7 @@ def too_many_values_to_unpack(message, frame, tb_data):
 # TODO: complete the work below; note noqa to be removed
 
 
-@add_message_parser
+@parser.add
 def invalid_literal_for_int(message, *_args):
     _ = current_lang.translate
     pattern = re.compile(r"invalid literal for int\(\) with base (\d+): '(.*)'")
@@ -177,3 +153,56 @@ def _convert_to_float(value):
         "before the result can be converted into an integer using `int()`.\n"
     ).format(value=value)
     return {"cause": cause, "suggest": hint}
+
+
+@parser.add
+def date_month_must_be_between_1_and_12(message, *_args):
+    _ = current_lang.translate
+
+    if message != "month must be in 1..12":
+        return {}
+
+    hint = _("Did you specify an invalid month?\n")
+    cause = _(
+        "I am guessing that you specify an invalid value for a month\n"
+        "in a `date` object. Valid values are integers, from 1 to 12.\n"
+    )
+    return {"cause": cause, "suggest": hint}
+
+
+def unrecognized_message(_value, frame, tb_data):
+    """This attempts to provide some help when a message is not recognized."""
+    _ = current_lang.translate
+    bad_line = tb_data.bad_line.strip()
+    if bad_line.startswith("raise ") or bad_line.startswith("raise\t"):
+        try:
+            name = inspect.getframeinfo(frame).function
+            fn_obj = frame.f_globals[name]
+        except Exception:
+            return {}
+    else:
+        all_objects = info_variables.get_all_objects(bad_line, frame)["name, obj"]
+        callables = []
+        for name, obj in all_objects:
+            if callable(obj):
+                callables.append((name, obj))
+        if not callables:
+            return {}
+
+        tokens = token_utils.get_significant_tokens(tb_data.bad_line)
+        name, fn_obj = callables[0]
+        if name != tokens[0]:
+            return {}
+
+    cause = _(
+        "I do not recognize this error message.\n"
+        "I am guessing that the problem is with the function `{name}`.\n"
+    ).format(name=name)
+
+    if hasattr(fn_obj, "__doc__") and fn_obj.__doc__ is not None:
+        cause += _("Its docstring is:\n\n`'''{docstring}'''`\n").format(
+            docstring=fn_obj.__doc__
+        )
+    else:
+        cause += _("I have no more information.\n")
+    return {"cause": cause}
