@@ -11,18 +11,25 @@ from .. import utils
 
 STATEMENT_ANALYZERS = []
 
+ASYNC = 0
+
+
+def more_errors():
+    _ = current_lang.translate
+    return "\n" + _(
+        "However, making such a change would still not correct\n"
+        "all the syntax errors in the code you wrote.\n"
+    )
+
 
 def def_correct_syntax():
-    # Note that a valid function definition must have at least 5 tokens.
-    # We make use of this in our analysis below, to ensure that we are
-    # making the proper identification of a mistake, as we go through the
-    # various possible cases.
     _ = current_lang.translate
+    async_ = "" if ASYNC == 0 else "async "
     # fmt: off
     return _(
         "The correct syntax is:\n\n"
-        "    def name ( ... ):"
-    ) + "\n"
+        "    {async_}def name ( ... ):"
+    ).format(async_=async_) + "\n"
     # fmt: on
 
 
@@ -31,11 +38,6 @@ def add_statement_analyzer(func):
     of all functions that analyze a single statement."""
     STATEMENT_ANALYZERS.append(func)
 
-    def wrapper(statement):
-        return func(statement)
-
-    return wrapper
-
 
 # ========================================================
 # Main calling function
@@ -45,16 +47,19 @@ def add_statement_analyzer(func):
 def analyze_def_statement(statement):
     """Analyzes the statement as identified by Python as that
     on which the error occurred."""
-    if not statement.tokens:
+    global ASYNC
+    if not statement.tokens:  # pragma: no cover
         debug_helper.log("Statement with no tokens in error_in_def.py")
         return {"cause": internal_error()}
 
+    ASYNC = 0
     if statement.tokens[0] == "async":
-        statement = remove_async(statement)
-        if statement is None:
-            return {}
+        ASYNC = 1
 
-    if str(statement.tokens[1]) in ("=", ":="):
+    if len(statement.tokens) > 1 + ASYNC and str(statement.tokens[1 + ASYNC]) in (
+        "=",
+        ":=",
+    ):
         # Let the generic method handle the wrong assignment case
         return {}
 
@@ -65,48 +70,12 @@ def analyze_def_statement(statement):
     return {}
 
 
-def remove_async(statement):
-    """To simplify the analysis, we replace any statement of the form
-    async def ...
-    by
-    def ...
-    """
-
-    if statement.tokens[0] != "async" or statement.tokens[1] != "def":
-        debug_helper.log("Problem in remove_async: inconsistent state")
-        return None
-
-    statement.tokens.pop(1)
-    statement.tokens[0].string = "def"
-    statement.bad_token_index -= 1
-    statement.nb_tokens -= 1
-    statement.prev_token = statement.tokens[statement.bad_token_index - 1]
-    try:
-        statement.next_token = statement.tokens[statement.bad_token_index + 1]
-    except IndexError:
-        pass  # undefined statement.next_token should have been already taken care of.
-
-    # We will want to make various text replacement, reconstruct (untokenize) the
-    # result to see if it is a valid statement. In doing so, we use the original
-    # logical line associated with a given token, and the row/col information.
-    # We need to make sure that the line content is consistent with the row/col
-    # information.  Thus, we want to replace
-    # async    def   something
-    # by
-    # def            something
-    for token in statement.tokens:
-        if token.line.strip().startswith("async"):
-            token.line = token.line.replace(" def", "", 1)
-            token.line = token.line.replace("async", "def      ", 1)
-    return statement
-
-
 @add_statement_analyzer
 def def_begin_code_block(statement):  #
     # Thinking of trying to use def to begin a code block, i.e.
     # def : ...
     _ = current_lang.translate
-    if statement.nb_tokens > 2 or statement.bad_token != ":":
+    if statement.nb_tokens > 2 + ASYNC or statement.bad_token != ":":
         return {}
 
     if statement.first_token.start_col == 0:
@@ -131,8 +100,8 @@ def missing_parens(statement):
 
     if (
         statement.bad_token != ":"
-        and statement.nb_tokens >= 3
-        and statement.bad_token != statement.tokens[2]
+        and statement.nb_tokens >= 3 + ASYNC
+        and statement.bad_token != statement.tokens[2 + ASYNC]
     ):
         return {}
 
@@ -157,7 +126,7 @@ def missing_parens_2(statement):
     # def test a, b:
     _ = current_lang.translate
 
-    if statement.bad_token_index != 2 and statement.last_token != ":":
+    if statement.bad_token_index != 2 + ASYNC and statement.last_token != ":":
         return {}
 
     new_statement = fixers.replace_two_tokens(
@@ -223,36 +192,13 @@ def missing_colon(statement):
     return {}
 
 
-@add_statement_analyzer
-def not_enough_tokens(statement):
-    _ = current_lang.translate
-
-    if statement.nb_tokens >= 5:
-        return {}
-
-    cause = _("You did not write a valid function definition.\n")
-    if statement.bad_token == "(" and statement.bad_token_index == 1:
-        new_statement = fixers.replace_token(
-            statement.tokens, statement.bad_token, "name"
-        )
-        if not fixers.check_statement(new_statement):
-            return {"cause": cause + def_correct_syntax()}
-
-        cause = hint = _("You forgot to name your function.\n")
-        return {"cause": cause + def_correct_syntax(), "suggest": hint}
-
-    return {"cause": cause + def_correct_syntax()}
-
-
 @add_statement_analyzer
 def keyword_as_function_name(statement):
     # Something like
-    # def pass() ...
+    # def pass(): ...
     _ = current_lang.translate
-    if not (
-        statement.bad_token.is_keyword()
-        and statement.prev_token == statement.first_token
-    ):
+    def_token = statement.tokens[ASYNC]
+    if not (statement.bad_token.is_keyword() and statement.prev_token == def_token):
         return {}
 
     hint = _("You cannot use a Python keyword as a function name.\n")
@@ -272,10 +218,8 @@ def keyword_as_function_name(statement):
 @add_statement_analyzer
 def other_invalid_function_names(statement):
     _ = current_lang.translate
-
-    if statement.bad_token.is_identifier() or not (
-        statement.prev_token == statement.first_token
-    ):
+    def_token = statement.tokens[ASYNC]
+    if statement.bad_token.is_identifier() or not (statement.prev_token == def_token):
         return {}
 
     new_statement = fixers.replace_token(
@@ -299,10 +243,11 @@ def other_invalid_function_names(statement):
 @add_statement_analyzer
 def function_definition_missing_name(statement):
     _ = current_lang.translate
+    def_token = statement.tokens[ASYNC]
     if not (
-        statement.first_token == "def"
+        def_token == "def"
         and statement.bad_token == "("
-        and statement.prev_token == statement.first_token
+        and statement.prev_token == def_token
     ):
         return {}
 
@@ -342,14 +287,19 @@ def keyword_not_allowed_as_function_argument(statement):
 @add_statement_analyzer
 def dotted_name_not_allowed(statement):
     _ = current_lang.translate
-    if statement.bad_token != ".":
+    if not (statement.bad_token == "." and statement.prev_token.is_identifier()):
         return {}
 
-    if statement.bad_token_index > 3:
+    if statement.bad_token_index > 3 + ASYNC:
         cause = _("You cannot use dotted names as function arguments.\n")
+        if statement.next_token.is_identifier():
+            cause += _("Perhaps you meant to write a comma.\n")
+            hint = _("Did you mean to write a comma?\n")
+        else:
+            hint = cause
     else:
-        cause = _("You cannot use dots in function names.\n")
-    return {"cause": cause, "suggest": cause}
+        hint = cause = _("You cannot use dots in function names.\n")
+    return {"cause": cause, "suggest": hint}
 
 
 @add_statement_analyzer
@@ -375,7 +325,7 @@ def positional_arguments_in_def(statement):
         return {"cause": cause, "suggest": hint}
 
     prev_tok = ""
-    for tok in statement.tokens[0 : statement.bad_token_index]:
+    for tok in statement.tokens[ASYNC : statement.bad_token_index]:
         if tok == "**":
             cause = meaning + _(
                 "You have unspecified keyword arguments that appear before\n"
@@ -414,24 +364,41 @@ def keyword_arguments_in_def(statement):
     if statement.bad_token != "*" or statement.prev_token != ",":
         return {}
 
-    for tok in statement.tokens[0 : statement.bad_token_index]:
+    args = statement.next_token if statement.next_token.is_identifier() else ""
+    hint = _("You can only use `*` once in a function definition.\n")
+    tokens = statement.tokens
+    for index, tok in enumerate(tokens[ASYNC : statement.bad_token_index]):
+        next_token = tokens[index + 1]
         if tok == "*":
-            hint = _("You can only use `*` once in a function definition.\n")
-            cause = hint + _(
-                "It must either be used by itself, `..., *, ...`,\n"
-                "or in the form `..., *args ...`, but not both.\n"
-            )
-            return {"cause": cause, "suggest": hint}
+            if next_token.is_identifier() and args:
+                cause = hint + _(
+                    "You have used it twice, with `*{first}` and `*{second}`.\n"
+                ).format(first=next_token, second=args)
+                return {"cause": cause, "suggest": hint}
+            elif next_token.is_identifier():
+                args = next_token.string
 
-        if str(tok) in ("**", "="):
-            if statement.next_token.is_identifier():
-                cause = _(
-                    "`*{name}` must appear before any keyword argument.\n"
-                ).format(name=statement.next_token.string)
+            if args:
+                cause = hint + _(
+                    "It must either be used by itself, `*`,\n"
+                    "or in the form `*{args}`, but not both.\n"
+                ).format(args=args)
             else:
-                cause = _("Keyword arguments must appear after the `*` operator.\n")
-            return {"cause": cause, "suggest": cause}
-    return {}
+                cause = hint
+            return {"cause": cause, "suggest": hint}
+        elif tok == "**":
+            if args:
+                cause = _("`*{args}` must appear before `**{kwargs}`.\n").format(
+                    args=args, kwargs=next_token
+                )
+            else:
+                cause = _("`**{kwargs}` must appear after the `*` operator.\n").format(
+                    kwargs=next_token
+                )
+            return {"cause": cause, "suggest": hint}
+    else:  # pragma: no cover
+        debug_helper.log("New case to consider for * as bad token.")
+        return {"cause": hint, "suggest": hint}
 
 
 @add_statement_analyzer
@@ -515,8 +482,6 @@ def operator_as_argument(statement):
     1. operator instead of equal sign
     """
     _ = current_lang.translate
-    # TODO: add tests
-
     if not statement.bad_token.is_operator() or statement.prev_token == "def":
         return {}
 
@@ -564,34 +529,39 @@ def operator_as_argument(statement):
         return {"cause": cause, "suggest": hint}
 
     # def test(a, **, b): -> def test(a, c, b):
-    if statement.prev_token.string in "(,":
+    if statement.prev_token.string in ("(", ","):
         # prevent getting an error because of repeated argument
         unique_name = utils.unique_variable_name()
         new_statement = fixers.replace_token(
             statement.statement_tokens, statement.bad_token, unique_name
         )
+        hint = _("You cannot use `{op}` as an argument.\n").format(
+            op=statement.bad_token
+        )
         if fixers.check_statement(new_statement):
-            hint = _("You cannot use `{op}` as an argument.\n").format(
-                op=statement.bad_token
-            )
             cause = _(
                 "I suspect you made a typo and wrote `{op}` by mistake.\n"
                 "If you replace it by a unique variable name, the result\n"
                 "will contain no syntax error.\n"
             ).format(op=statement.bad_token)
-            return {"cause": cause, "suggest": hint}
 
-    # The following has no corresponding unit test, but has been tested in a repl
-    if statement.prev_token.string in ("(", ","):
-        hint = _("You cannot use `{op}` as an argument.\n").format(
-            op=statement.bad_token
-        )
-        cause = hint + _(
-            "You can only use identifiers (variable names) as function arguments.\n"
-        )
+        else:
+            cause = (
+                _(
+                    "I suspect you made a typo and wrote `{op}` by mistake,\n"
+                    "perhaps instead of writing an identifier (variable name).\n"
+                )
+                + more_errors()
+            )
         return {"cause": cause, "suggest": hint}
 
-    return {}
+    if statement.prev_token == "**":
+        cause = _(
+            "The `**` operator needs to be followed by an identifier (variable name).\n"
+        )
+        return {"cause": cause}
+
+    return {}  # pragma: no cover
 
 
 @add_statement_analyzer
@@ -609,7 +579,7 @@ def arg_after_kwarg(statement):
     ):
         return {}
 
-    for tok in statement.tokens[0 : statement.bad_token_index]:
+    for tok in statement.tokens[ASYNC : statement.bad_token_index]:
         if tok == "**":
             hint = _("Positional arguments must come before keyword arguments.\n")
             cause = hint + _(
