@@ -53,7 +53,7 @@ def _modify_source(tokens, original_token, replace="", append="", prepend=""):
     leading and ending spaces so that it could be inserted in a
     code sample at the beginning of a line with no indentation.
     """
-    if not tokens:
+    if not tokens:  # pragma: no cover
         debug_helper.log("Problem in fixers._modify_source().")
         debug_helper.log("Empty token list was received")
         debug_helper.log_error()
@@ -76,7 +76,7 @@ def _modify_source(tokens, original_token, replace="", append="", prepend=""):
                 new_tokens.append(tok)
         source = token_utils.untokenize(new_tokens)
         return source.strip()
-    except Exception as e:
+    except Exception as e:  # pragma: no cover
         debug_helper.log("Problem in fixers._modify_source().")
         debug_helper.log_error(e)
         return token_utils.untokenize(tokens)
@@ -86,7 +86,7 @@ def replace_two_tokens(
     tokens, first_token, first_string="", second_token=None, second_string=""
 ):
     """Replace two tokens at once by their new string value"""
-    if second_token is None:
+    if second_token is None:  # pragma: no cover
         debug_helper.log("Problem in fixers.replace_two_tokens()")
         debug_helper.log("second_token should not be None")
         return token_utils.untokenize(tokens)
@@ -106,7 +106,7 @@ def replace_two_tokens(
 
         source = token_utils.untokenize(new_tokens)
         return source.strip()
-    except Exception as e:
+    except Exception as e:  # pragma: no cover
         debug_helper.log("Problem in fixers.replace_two_tokens().")
         debug_helper.log_error(e)
         return token_utils.untokenize(tokens)
@@ -142,7 +142,7 @@ def check_statement(statement):
         except SyntaxError:
             return False
 
-    except Exception as e:
+    except Exception as e:  # pragma: no cover
         debug_helper.log("Problem in token_utils.check_statement().")
         debug_helper.log_error(e)
         return False
@@ -164,55 +164,3 @@ try:
     pass
 %s
 """
-
-
-def calculate_token_shift(old_statement, new_statement):
-    """Given two statements that contain a SyntaxError, the position
-    of the token causing the error is calculated in both cases; by position,
-    we mean the index of the token when a statement is converted into
-    a list of meaningful token.
-
-    The original bad token is returned as well as the
-    difference in position (new - old).
-    """
-    # Note: this is currently unused.
-    old_tokens = token_utils.get_significant_tokens(old_statement)
-    try:
-        compile(old_statement, "fake-file", "exec")
-        debug_helper.log("Problem in calculate_token_shift()")
-        debug_helper.log("No error found in old_statement")
-        return -1
-    except SyntaxError as e:
-        row = e.lineno
-        offset = e.offset
-        bad_token, old_index = find_token_by_index(old_tokens, row, offset)
-
-    new_tokens = token_utils.get_significant_tokens(new_statement)
-    try:
-        compile(new_statement, "fake-file", "exec")
-        debug_helper.log("Problem in calculate_token_shift()")
-        debug_helper.log("No error found in new_statement")
-        return -1
-    except SyntaxError as e:
-        row = e.lineno
-        offset = e.offset
-        _ignore, new_index = find_token_by_index(new_tokens, row, offset)
-
-    return bad_token, new_index - old_index
-
-
-def find_token_by_index(tokens, row, column):
-    """Given a list of tokens, a specific row (linenumber) and column (offset),
-    the token itself, as well as the list index of the token found
-    at that position is returned.
-
-    If no such token is found, None, -1 is returned
-    """
-    for index, tok in enumerate(tokens):
-        if (
-            tok.start_row <= row <= tok.end_row
-            and tok.start_col <= column < tok.end_col
-        ):
-            return tok, index
-    debug_helper.log("problem in find_token_by_index: token not found")
-    return None, -1
