@@ -11,9 +11,10 @@ import sys
 from . import fixers
 from . import syntax_utils
 from . import statement_analyzer
+from . import error_in_def
 from .. import debug_helper
 from .. import utils
-from ..my_gettext import current_lang, use_www
+from ..my_gettext import current_lang
 
 MESSAGE_ANALYZERS = []
 
@@ -82,16 +83,41 @@ def assign_to_keyword(message="", statement=None):
         "cannot use assignment expressions with False",  # Python 3.8
         "cannot use assignment expressions with None",  # Python 3.8
         "cannot use assignment expressions with Ellipsis",  # Python 3.8
+        "cannot assign to Ellipsis here. Maybe you meant '==' instead of '='?",
+        "cannot assign to ellipsis here. Maybe you meant '==' instead of '='?",
     ):
         return {}
 
-    word = statement.bad_token
-    hint = _("You cannot assign a value to `{keyword}`.").format(keyword=word)
-    if word in ["None", "True", "False", "__debug__", "Ellipsis (...)"]:
-        cause = _(
-            "`{keyword}` is a constant in Python; you cannot assign it a value.\n" "\n"
-        ).format(keyword=word)
+    for word in ["None", "True", "False", "__debug__", "Ellipsis", "ellipsis"]:
+        if word in message:
+            if word == "ellipsis":
+                word = "Ellipsis"
+            break
     else:
+        if statement.bad_token.is_keyword():
+            word = statement.bad_token
+        else:  # something like name.constant = ?
+            for tok in statement.tokens[statement.bad_token_index :]:
+                if tok.is_keyword():
+                    word = tok
+                    break
+            else:  # pragma: no cover
+                debug_helper.log(f"Case not covered: {statement.bad_line}")
+                return {}
+    hint = _("You cannot assign a value to `{keyword}`.").format(keyword=word)
+
+    if word == "Ellipsis":
+        hint = _("You cannot assign a value to the ellipsis symbol [`...`].\n")
+        cause = _(
+            "The ellipsis symbol `...` is a constant in Python;"
+            "you cannot assign it a different value.\n"
+        ).format(keyword=word)
+    elif word in ["None", "True", "False", "__debug__"]:
+        cause = _(
+            "`{keyword}` is a constant in Python; you cannot assign it a different value.\n"
+        ).format(keyword=word)
+    else:  # pragma: no cover
+        debug_helper.log(f"Case not covered: {statement.bad_line}")
         cause = _(
             "You were trying to assign a value to the Python keyword `{keyword}`.\n"
             "This is not allowed.\n"
@@ -237,7 +263,8 @@ def what_kind_of_literal(literal):
         if isinstance(a, kind):
             return result
 
-    return None
+    debug_helper.log("New kind of literal" + str(a))  # pragma: no cover
+    return None  # pragma: no cover
 
 
 @add_python_message
@@ -276,7 +303,8 @@ def assign_to_literal(message="", statement=None):
     tokens = statement.tokens[0 : statement.bad_token_index]
     for_loop = False
     for tok in tokens[::-1]:
-        if tok == "in":
+        if tok == "in":  # pragma: no cover
+            debug_helper.log("New case for assign_to_literal")
             break
         elif tok == "for":
             for_loop = True
@@ -376,7 +404,7 @@ def assign_to_operator(message="", statement=None):
         ).format(name=name, original=name.replace("_", "-"))
         return {"cause": cause, "suggest": hint}
 
-    hint = _("You can only assign objects to identifiers (variable names).\n")
+    hint = _("Perhaps you needed `==` instead of `=`.\n")
     return {"cause": cause, "suggest": hint}
 
 
@@ -389,8 +417,8 @@ def could_be_identifier(line):
                 if lhs.isidentifier():
                     return lhs
         return ""
-    except Exception:  # noqa
-        print("exception raised")
+    except Exception as e:  # pragma: no cover
+        debug_helper.log("Problem in could_be_idenfier:" + str(e))
         return ""
 
 
@@ -650,18 +678,20 @@ def mismatched_parenthesis(message="", statement=None):
     if cause:
         return cause
 
-    if lineno is not None:
-        cause = _(
-            "Python tells us that the closing `{closing}` on the last line shown\n"
-            "does not match the opening `{opening}` on line {lineno}.\n\n"
-        ).format(closing=closing, opening=opening, lineno=lineno)
-    else:
-        cause = _(
-            "Python tells us that the closing `{closing}` on the last line shown\n"
-            "does not match the opening `{opening}`.\n\n"
-        ).format(closing=closing, opening=opening)
+    if True:  # pragma: no cover
+        debug_helper.log("statement_analyzer.mismatched_brackets failed.")
+        if lineno is not None:
+            cause = _(
+                "Python tells us that the closing `{closing}` on the last line shown\n"
+                "does not match the opening `{opening}` on line {lineno}.\n\n"
+            ).format(closing=closing, opening=opening, lineno=lineno)
+        else:
+            cause = _(
+                "Python tells us that the closing `{closing}` on the last line shown\n"
+                "does not match the opening `{opening}`.\n\n"
+            ).format(closing=closing, opening=opening)
 
-    return {"cause": cause}
+        return {"cause": cause}
 
 
 @add_python_message
@@ -684,7 +714,8 @@ def unterminated_f_string(message="", statement=None):
         ):
             fstring = tok
             break
-    else:
+    else:  # pragma: no cover
+        debug_helper.log("Need to record case in unterminated_f_string")
         fstring = "<not found>"
     cause = _(
         "Inside the f-string `{fstring}`, \n"
@@ -705,7 +736,8 @@ def name_is_parameter_and_global(message="", statement=None):
     name = message.split("'")[1]
     if name in line and "global" in line:
         newline = line
-    else:
+    else:  # pragma: no cover
+        debug_helper.log("New case for name_is_parameter_and_global")
         newline = f"global {name}"
     cause = _(
         "You are including the statement\n\n"
@@ -845,7 +877,7 @@ def unexpected_character_after_continuation(message="", statement=None):
             if found_continuation:
                 bad_token = tok
                 break
-        else:
+        else:  # pragma: no cover
             debug_helper.log("Could not find bad token after continuation character.")
 
     if bad_token.is_number():
@@ -944,7 +976,7 @@ def python2_print(message="", **_kwargs):
         return {}
     message = message[59:-2]
     if len(message) > 40:
-        message = message[0:15] + " ... "
+        message = message[0:25] + " ... "
     cause = _(
         "Perhaps you need to type\n\n"
         "     print({message})\n\n"
@@ -998,6 +1030,7 @@ def too_many_nested_blocks(message="", **_kwargs):
 @add_python_message
 def named_arguments_must_follow_bare_star(message="", **_kwargs):
     _ = current_lang.translate
+    # TODO: revise this as it can be greatly improved
     if message != "named arguments must follow bare *":
         return {}
 
@@ -1011,9 +1044,9 @@ def named_arguments_must_follow_bare_star(message="", **_kwargs):
 
 
 @add_python_message
-def you_found_it(message="", **_kwargs):
+def you_found_it(message="", statement=None):  # pragma: no cover
     _ = current_lang.translate
-    if message != "You found it!":
+    if message != "You found it!" or statement.bad_token != "__peg_parser__":
         return {}
 
     cause = _(
@@ -1119,10 +1152,13 @@ def eof_unclosed_triple_quoted(message="", **_kwargs):
 def proper_decimal_or_octal_number(prev_str, bad_str):
     # see next two cases
     _ = current_lang.translate
-    if not (set(prev_str).issubset("_0") and prev_str.startswith("0")):  # noqa
+    if not (
+        set(prev_str).issubset("_0") and prev_str.startswith("0")
+    ):  # pragma: no cover
+        debug_helper.log("proper_decimal_or_octal_number should not have been called")
         return {}
 
-    if prev_str == "0" and set(bad_str).issubset("01234567_"):  # noqa
+    if prev_str == "0" and set(bad_str).issubset("01234567_"):
         correct = "0o" + bad_str
         hint = _("Did you mean `{num}`?\n").format(num=correct)
         cause = _(
@@ -1141,7 +1177,7 @@ def proper_decimal_or_octal_number(prev_str, bad_str):
         ).format(num=correct)
         return {"cause": cause, "suggest": hint}
 
-    return {}
+    return {}  # pragma: no cover
 
 
 @add_python_message
@@ -1198,7 +1234,7 @@ def parens_around_exceptions(message="", **_kwargs):
     # keep in sync with statement_analyzer.parens_around_exceptions
     _ = current_lang.translate
 
-    if message != "exception group must be parenthesized":
+    if message != "multiple exception types must be parenthesized":
         return {}
 
     hint = _("Did you forget parentheses?\n")
@@ -1207,14 +1243,7 @@ def parens_around_exceptions(message="", **_kwargs):
         "with multiple exception types. If that is the case, you must\n"
         "surround them with parentheses.\n"
     )
-    python_link = _(
-        "https://docs.python.org/3/tutorial/errors.html#handling-exceptions"
-    )
-    return {
-        "cause": cause + "\n" + use_www(),
-        "suggest": hint,
-        "python_link": python_link,
-    }
+    return {"cause": cause + "\n", "suggest": hint}
 
 
 @add_python_message
@@ -1245,7 +1274,8 @@ def colon_expected(message="", statement=None):
     new_statement = fixers.modify_token(
         statement.statement_tokens, statement.bad_token, append=":"
     )
-    if fixers.check_statement(new_statement):
+    if fixers.check_statement(new_statement):  # pragma: no cover
+        debug_helper.log("New case for colon_expected.")
         return {"cause": cause, "suggest": hint}
 
     return {}
@@ -1288,3 +1318,60 @@ def invalid_double_star_operator(message="", **_kwargs):
         return {"cause": cause}
 
     return {}
+
+
+@add_python_message
+def invalid_hexadecimal_literal(message="", statement=None):
+    _ = current_lang.translate
+
+    if message != "invalid hexadecimal literal":  # new in Python 3.10
+        return {}
+    if not statement.highlighted_tokens:
+        statement.highlighted_tokens = [statement.bad_token, statement.next_token]
+
+    if statement.first_token == "def" or (
+        statement.first_token == "async" and statement.tokens[1] == "def"
+    ):
+        cause = error_in_def.analyze_def_statement(statement)
+        if cause:
+            return cause
+
+    return statement_analyzer.invalid_hexadecimal(statement)
+
+
+@add_python_message
+def invalid_decimal_literal(message="", statement=None):
+    _ = current_lang.translate
+
+    if message != "invalid decimal literal":  # new in Python 3.10
+        return {}
+    if not statement.highlighted_tokens:
+        statement.highlighted_tokens = [statement.bad_token, statement.next_token]
+
+    if statement.first_token == "def" or (
+        statement.first_token == "async" and statement.tokens[1] == "def"
+    ):
+        cause = error_in_def.analyze_def_statement(statement)
+        if cause:
+            return cause
+
+    return statement_analyzer.invalid_name(statement)
+
+
+@add_python_message
+def invalid_imaginary_literal(message="", statement=None):
+    _ = current_lang.translate
+
+    if message != "invalid imaginary literal":  # new in Python 3.10
+        return {}
+    if not statement.highlighted_tokens:
+        statement.highlighted_tokens = [statement.bad_token, statement.next_token]
+
+    if statement.first_token == "def" or (
+        statement.first_token == "async" and statement.tokens[1] == "def"
+    ):
+        cause = error_in_def.analyze_def_statement(statement)
+        if cause:
+            return cause
+
+    return statement_analyzer.invalid_name(statement)
