@@ -6,7 +6,6 @@ from ..source_cache import cache
 from .syntax_utils import matching_brackets
 from .. import debug_helper
 from .. import token_utils
-from ..my_gettext import internal_error
 
 
 # During the analysis for finding the cause of the error, we typically examine
@@ -17,6 +16,7 @@ from ..my_gettext import internal_error
 # neighbours, the code for the analysis can be greatly simplified as we do
 # not have to verify the existence of these neighbours.
 MEANINGLESS_TOKEN = token_utils.tokenize(" ")[0]
+TOO_MANY_BLOCKS = "too many statically nested blocks"
 
 
 class Statement:
@@ -51,6 +51,19 @@ class Statement:
         self.message = value.msg
         self.offset = value.offset
 
+        # Python 3.10 introduced new attributes for 'value'.
+        # We already have taken care of assigning some default values
+        # to it in core.py.
+        self.end_offset = value.end_offset
+        self.end_linenumber = value.end_lineno
+
+        if self.end_offset is None or self.offset is None:
+            self.highlighted_tokens = None
+        elif self.end_offset - self.offset == 1:
+            self.highlighted_tokens = None
+        else:
+            self.highlighted_tokens = []
+
         # From the traceback, we were previously able ot obtain the line
         # of code identified by Python as being problematic.
         self.bad_line = bad_line  # previously obtained from the traceback
@@ -89,9 +102,12 @@ class Statement:
         self.using_friendly_console = False
         if self.filename is not None:
             self.using_friendly_console = self.filename.startswith("<friendly")
-        elif "too many statically nested blocks" not in self.message:
-            # We know of only one case where filename should be None.
+        elif (
+            TOO_MANY_BLOCKS not in self.message
+            and "encoding problem" not in self.message
+        ):  # pragma: no cover
             debug_helper.log("filename is None in source_info.Statement")
+            debug_helper.log("Add this as a new test.")
 
         self.get_token_info()
 
@@ -106,11 +122,10 @@ class Statement:
             self.obtain_statement(source_tokens)
             self.tokens = self.remove_meaningless_tokens()
             if not self.tokens:
-                if len(self.all_statements) > 1:
+                if len(self.all_statements) > 1:  # pragma: no cover
                     self.statement_tokens = self.all_statements[-2]
                     self.tokens = self.remove_meaningless_tokens()
                 else:
-                    print(internal_error())
                     self.tokens = [token_utils.tokenize("Internal_error")[0]]
 
             self.statement = token_utils.untokenize(self.statement_tokens)
@@ -118,7 +133,7 @@ class Statement:
                 self.filename.startswith("<friendly-console")
                 and self.statement_brackets
                 and not self.end_bracket
-            ):
+            ):  # pragma: no cover
                 # We got an error flagged before we had the chance to close
                 # brackets. Unclosed brackets should not be a problem on their
                 # own in a console session - so, we make sure to close
@@ -153,12 +168,19 @@ class Statement:
                 last_token.string = last_token.line = add_token
                 self.tokens.append(last_token)
 
-        elif "too many statically nested blocks" not in self.message:
+        elif (
+            TOO_MANY_BLOCKS not in self.message
+            and "encoding problem" not in self.message
+        ):  # pragma: no cover
             debug_helper.log("linenumber is None in source_info.Statement")
+            debug_helper.log("Add this as new test case")
 
         if self.tokens:
             self.assign_individual_token_values()
-        elif "too many statically nested blocks" not in self.message:
+        elif (
+            TOO_MANY_BLOCKS not in self.message
+            and "encoding problem" not in self.message
+        ):  # pragma: no cover
             debug_helper.log("No meaningful tokens in source_info.Statement")
 
     def get_source_tokens(self):
@@ -197,7 +219,8 @@ class Statement:
 
             if self.bad_token_index == 0:
                 self.prev_token = MEANINGLESS_TOKEN
-            elif self.prev_token is None:
+            elif self.prev_token is None:  # pragma: no cover
+                debug_helper.log("This case should be added as new test.")
                 self.prev_token = self.tokens[self.bad_token_index - 1]
 
         if self.last_token != self.bad_token:
@@ -279,57 +302,33 @@ class Statement:
         """
 
         previous_row = -1
-        previous_row_non_space = -1
         previous_token = None
         continuation_line = False
-        last_closing = None
+        # Some tokens cannot occur within brackets; if they are indicated as being
+        # the offending token, it might be because we have an unclosed bracket.
+        should_begin_statement = [
+            "async",
+            "await",
+            "class",
+            "def",
+            "return",
+            "elif",
+            "import",
+            "try",
+            "except",
+            "finally",
+            "with",
+            "while",
+            "yield",
+        ]
 
         for token in source_tokens:
-            # Did we collect all the tokens belonging to the bad statement?
-            if (  # is the bad token identified as the very first token of a new line
-                token.string.strip()
-                and token.start_row == self.linenumber
-                and token.start_col <= self.offset <= token.end_col
-                and previous_token is not None
-                and previous_token.string.strip()
-                and previous_token.start_row < token.start_row
-                and token.string not in ")]}"
-            ):
-                break
             if (
                 token.start_row > self.linenumber
-                and (last_closing is None or token.start_row > last_closing.start_row)
                 and not continuation_line
+                and not self.statement_brackets
             ):
-                if not self.statement_brackets:
-                    break
-                # perhaps we have some unclosed bracket causing the error
-                # and we meant to start a new statement
-                if token.string in (
-                    "async",
-                    "await",
-                    "class",
-                    "def",
-                    "return",
-                    "elif",
-                    "import",
-                    "try",
-                    "except",
-                    "finally",
-                    "with",
-                    "while",
-                    "yield",
-                    # ";",
-                ):
-                    break
-
-                if self.bad_token == ":" or (
-                    token.string in ("if", "else", "for")
-                    and token.start_row > previous_row_non_space
-                ):
-                    break
-            if token.string.strip():
-                previous_row_non_space = token.end_row
+                break
 
             # is this a new statement?
             # Valid statements will have matching brackets (), {}, [].
@@ -349,7 +348,6 @@ class Statement:
                         self.all_statements.append(self.statement_tokens[:])
                     self.statement_tokens = []
                     self.begin_brackets = []
-                    last_closing = None
                 previous_row = token.start_row
 
             self.statement_tokens.append(token)
@@ -357,7 +355,20 @@ class Statement:
             # sometimes matching the beginning of a token, sometimes the end.
             # Furthermore, the end of a token (end_col) might be equal to
             # the beginning of the next (start_col).
-            if (
+            # Additionally, for Python 3.10, multiple tokens can be highlighted
+            # if they are on the same line.
+            if self.highlighted_tokens:  # not None and not empty list
+                if (
+                    self.linenumber == token.start_row
+                    and self.end_offset is not None
+                    and (
+                        self.offset < token.end_col < self.end_offset
+                        or self.end_offset == 0
+                    )
+                    and token.string.strip()
+                ):
+                    self.highlighted_tokens.append(token)
+            elif (
                 token.start_row == self.linenumber
                 and token.start_col <= self.offset <= token.end_col
                 and self.bad_token is None
@@ -366,6 +377,8 @@ class Statement:
                 self.bad_token = token
                 if self.bad_token.is_comment():
                     self.bad_token = self.prev_token
+                if self.highlighted_tokens is not None:
+                    self.highlighted_tokens.append(self.bad_token)
             elif (
                 token.string.strip()
                 and not token.is_comment()
@@ -374,6 +387,10 @@ class Statement:
                 self.prev_token = token
 
             previous_token = token
+
+            if self.bad_token in should_begin_statement and self.statement_brackets:
+                break  # we almost certainly have an unclosed bracket
+            # Note: '' in 'any string' == True
             # careful to not accidentally include null strings as brackets
             if not token.string or token.string not in "()[]}{":
                 continue
@@ -384,8 +401,6 @@ class Statement:
                     self.begin_brackets.append(token)
             elif token.string in ")]}":
                 self.end_bracket = token
-                last_closing = token
-
                 if not self.statement_brackets:
                     break
 
