@@ -8,7 +8,7 @@ import sys
 from . import error_in_def
 from . import fixers
 from . import syntax_utils
-from ..my_gettext import current_lang, internal_error, use_www
+from ..my_gettext import current_lang, internal_error
 from .. import debug_helper
 from .. import token_utils
 from .. import utils
@@ -29,6 +29,10 @@ def add_statement_analyzer(func):
     of all functions that analyze a single statement."""
     STATEMENT_ANALYZERS.append(func)
 
+    # The following is needed if we wish to call explicitly
+    # one of the functions below from another file.
+    # We sometimes do this for consistency when later version of
+    # Python give us a specific message instead of just 'invalid syntax'.
     def wrapper(statement):
         return func(statement)
 
@@ -51,7 +55,6 @@ def analyze_statement(statement):
         statement.first_token == "async" and statement.tokens[1] == "def"
     ):
         cause = error_in_def.analyze_def_statement(statement)
-        # The above call will have removed any "async" tokens for further analysis.
         if cause:
             return cause
 
@@ -130,18 +133,34 @@ def mismatched_brackets(statement):
 def copy_pasted_code(statement):
     """Detecting code that starts with a Python prompt"""
     _ = current_lang.translate
-    if statement.nb_tokens < 2:
+
+    first_token = statement.first_token
+    if first_token not in [">>", "..."]:
         return {}
 
-    tokens = statement.tokens
-    if tokens[0] == ">>" and tokens[1] == ">":
+    hint = _("Did you use copy-paste?\n")
+    if (
+        first_token == ">>"
+        and first_token == statement.bad_token
+        and statement.next_token == ">"
+    ):
         cause = _(
             "It looks like you copy-pasted code from an interactive interpreter.\n"
             "The Python prompt, `>>>`, should not be included in your code.\n"
         )
-        hint = _("Did you use copy-paste?\n")
         return {"cause": cause, "suggest": hint}
-    return {}
+    elif first_token == "...":
+        if statement.highlighted_tokens is not None:  # Python 3.10
+            is_ellipsis = statement.bad_token
+        else:
+            is_ellipsis = statement.prev_token
+        if is_ellipsis == first_token:
+            cause = _(
+                "It looks like you copy-pasted code from an interactive interpreter.\n"
+                "The Python prompt, `...`, should not be included in your code.\n"
+            )
+            return {"cause": cause, "suggest": hint}
+    return {}  # pragma: no cover
 
 
 @add_statement_analyzer
@@ -217,8 +236,6 @@ def keyword_as_attribute(statement):
         cause = _(
             "You cannot use the Python keyword `{word}` as an attribute.\n\n"
         ).format(word=word)
-    elif word == "__debug__":
-        cause = _("You cannot use the constant `__debug__` as an attribute.\n\n")
     else:
         return {}
 
@@ -282,14 +299,6 @@ def misplaced_quote(statement):
 
     bad_token = statement.bad_token
     if bad_token.is_identifier():
-        bad_string = statement.prev_token.string
-        for fn in [int, float, complex]:
-            try:
-                fn(bad_string)  # Definitely not a word!
-                return {}
-            except Exception:  # noqa
-                pass
-
         hint = _("Perhaps you misplaced a quote.\n")
         cause = _(
             "There appears to be a Python identifier (variable name)\n"
@@ -356,7 +365,10 @@ def inverted_operators(statement):
         is_op(next_)
         and is_op(next_.string + bad.string)
         and bad.immediately_before(next_)
-    ):
+    ):  # pragma: no cover
+        # I cannot think of a situation where Python would highlight the first
+        # of two consecutive operators as being the incorrect one.
+        debug_helper.log("inverted_operators: new case")
         first = bad
         second = next_
     else:
@@ -418,63 +430,38 @@ def consecutive_operators(statement):
             "or forgot to write something between them.\n"
         ).format(first=statement.prev_token, second=statement.bad_token)
 
+    if is_op(statement.prev_token.string + statement.bad_token.string):
+        cause += _(
+            "Or perhaps you included a space by mistake between the two operators\n"
+            "and meant to write `{oper}` as a single operator.\n"
+        ).format(oper=statement.prev_token.string + statement.bad_token.string)
     return {"cause": cause}
 
 
-@add_statement_analyzer
-def space_between_operators(statement):
-    """Detect if some space has been inserted between two operators"""
+def _walrus_instead_of_equal_39(statement):
+    # Python version 3.9 identifies a token beyond :=
     _ = current_lang.translate
-
-    is_op = token_utils.is_operator
-    if not is_op(statement.bad_token):
-        return {}
-
-    prev = statement.prev_token
-    bad = statement.bad_token
-    next_ = statement.next_token
-
-    if is_op(prev) and is_op(prev.string + bad.string):
-        first_token = prev
-        second_token = bad
-    elif is_op(next_) and is_op(bad.string + next_.string):
-        first_token = bad
-        second_token = next_
+    for tok in statement.statement_tokens[: statement.bad_token_index + 1]:
+        if tok == ":=":
+            return tok
     else:
-        return {}
-
-    hint = _("Did you leave some spaces between operators?\n")
-    possible_cause = _(
-        "It looks like you wrote two operators, `{first}` and `{second}`,\n"
-        "separated by spaces instead of writing them as a single operator:\n"
-        "`{correct}`"
-    )
-    correct = first_token.string + second_token.string
-    cause = possible_cause.format(
-        first=bad.string, second=next_.string, correct=correct
-    )
-    new_statement = fixers.replace_two_tokens(
-        statement.statement_tokens,
-        first_token,
-        first_string=correct,
-        second_token=second_token,
-        second_string="",
-    )
-    if fixers.check_statement(new_statement):
-        return {"cause": cause, "suggest": hint}
-
-    return {"cause": cause + more_errors(), "suggest": hint}
+        return None
 
 
 @add_statement_analyzer
 def walrus_instead_of_equal(statement):
     _ = current_lang.translate
-    if statement.bad_token != ":=":
-        return {}
 
-    new_statement = fixers.replace_token(
-        statement.statement_tokens, statement.bad_token, "="
-    )
+    if (3, 8) < sys.version_info < (3, 10):
+        bad_token = _walrus_instead_of_equal_39(statement)
+        if bad_token is None:
+            return {}
+    elif statement.bad_token != ":=":
+        return {}
+    else:
+        bad_token = statement.bad_token
+
+    new_statement = fixers.replace_token(statement.statement_tokens, bad_token, "=")
     if fixers.check_statement(new_statement):
         hint = _("Did you mean to use `=`?\n")
         cause = _(
@@ -491,24 +478,24 @@ def assign_instead_of_equal(statement):
     an equal sign, '==', in an if, elif or while statement."""
     _ = current_lang.translate
 
-    if statement.bad_token != "=":
+    if statement.highlighted_tokens:  # Python 3.10
+        bad_token = statement.next_token
+    else:
+        bad_token = statement.bad_token
+
+    if bad_token != "=":
         return {}
 
     if statement.first_token.string not in ("if", "elif", "while"):
         return {}
 
-    new_statement = fixers.replace_token(
-        statement.statement_tokens, statement.bad_token, "=="
-    )
+    new_statement = fixers.replace_token(statement.statement_tokens, bad_token, "==")
     if not fixers.check_statement(new_statement):
-        debug_helper.log("Fix did not work in assign_instead_of_equal")
         additional_cause = more_errors()
     else:
         additional_cause = ""
 
-    new_statement = fixers.replace_token(
-        statement.statement_tokens, statement.bad_token, ":="
-    )
+    new_statement = fixers.replace_token(statement.statement_tokens, bad_token, ":=")
 
     if sys.version_info < (3, 8) or not fixers.check_statement(new_statement):
         hint = _("Perhaps you needed `==` instead of `=`.\n")
@@ -517,47 +504,60 @@ def assign_instead_of_equal(statement):
         )
         return {"cause": cause + additional_cause, "suggest": hint}
 
-    if statement.next_token == ":":
-        hint = _("Perhaps you needed `:=` instead of `=:`.\n")
-        cause = _(
-            "You used an assignment operator `=` followed by `:`; perhaps you meant to use \n"
-            "the walrus operator `:=`.\n"
-        )
-    else:
-        hint = _("Perhaps you needed `==` or `:=` instead of `=`.\n")
-        cause = _(
-            "You used an assignment operator `=`; perhaps you meant to use \n"
-            "an equality operator, `==`, or the walrus operator `:=`.\n"
-        )
+    hint = _("Perhaps you needed `==` or `:=` instead of `=`.\n")
+    cause = _(
+        "You used an assignment operator `=`; perhaps you meant to use \n"
+        "an equality operator, `==`, or the walrus operator `:=`.\n"
+    )
     return {"cause": cause + additional_cause, "suggest": hint}
 
 
 @add_statement_analyzer
 def print_as_statement(statement):
+    # example: print len('hello')
     _ = current_lang.translate
 
-    if statement.prev_token == "print" and statement.bad_token != "(":
-        cause = _(
-            "In older version of Python, `print` was a keyword.\n"
-            "Now, `print` is a function; you need to use parentheses to call it.\n"
-        )
-        bad_line = statement.bad_line
-        if bad_line.count("(") == bad_line.count(")"):
-            new_line = bad_line.replace("print", "", 1).strip()
-            if len(new_line) > 30:
-                new_line = new_line[0:10] + " ... " + new_line[-10:]
-            new_line = "print(" + new_line + ")"
-        else:
-            new_line = "print(...)"
-        hint = _("Did you mean `{new_line}`?\n").format(new_line=new_line)
-        return {"cause": cause, "suggest": hint}
-    return {}
+    if (  # Python 3.10
+        statement.bad_token == statement.first_token == "print"
+        and statement.highlighted_tokens is not None
+    ):
+        pass
+    elif (  # Python < 3.10
+        statement.prev_token == statement.first_token == "print"
+        and statement.bad_token != "("
+    ):
+        pass
+    else:
+        return {}
+
+    cause = _(
+        "In older version of Python, `print` was a keyword.\n"
+        "Now, `print` is a function; you need to use parentheses to call it.\n"
+    )
+    bad_line = statement.bad_line
+    if bad_line.count("(") == bad_line.count(")"):
+        new_line = bad_line.replace("print", "", 1).strip()
+        if len(new_line) > 30:
+            parts = new_line.split(" ")
+            new_line = parts[0] + " ... " + parts[-1]
+        new_line = "print(" + new_line + ")"
+    else:
+        new_line = "print(...)"
+    hint = _("Did you mean `{new_line}`?\n").format(new_line=new_line)
+    return {"cause": cause, "suggest": hint}
 
 
 @add_statement_analyzer
 def calling_python_or_pip(statement):
     _ = current_lang.translate
-    if statement.first_token.string not in ("pip", "python"):
+    if statement.first_token.string not in ("pip", "python", "python3"):
+        return {}
+    # A single token statement with 'python' or 'pip' should not have
+    # caused a SyntaxError
+    second = statement.tokens[1]
+    # python followed by an operator is likely a valid statement.
+    # One exception is something like python -im friendly  :)
+    if second.is_operator() and second != "-":
         return {}
 
     cause = _(
@@ -631,13 +631,15 @@ def missing_colon(statement):
     """look for missing colon at the end of statement"""
     _ = current_lang.translate
 
+    # TODO: check all keywords listed here, with single keyword missing colon, like:
+    # if
+
     if statement.last_token == ":" or statement.bad_token != statement.last_token:
         return {}
 
     name = statement.first_token
-    # Note: "async def" statements are transformed into "def" statements
-    # prior to reaching this stage.
     if name.string not in (
+        "async",
         "class",
         "def",
         "if",
@@ -670,33 +672,22 @@ def missing_colon(statement):
         ).format(for_while=name)
         return {"cause": cause, "suggest": hint}
 
-    if name.string in ("def", "elif", "else", "except", "finally", "if", "try", "with"):
-        cause = _(
-            "You wrote a statement beginning with\n"
-            "`{name}` but forgot to add a colon `:` at the end.\n"
-            "\n"
-        ).format(name=name)
-        return {"cause": cause, "suggest": hint}
-
-    return {}
+    cause = _(
+        "You wrote a statement beginning with\n"
+        "`{name}` but forgot to add a colon `:` at the end.\n"
+        "\n"
+    ).format(name=name)
+    return {"cause": cause, "suggest": hint}
 
 
 @add_statement_analyzer
-def semi_colon_instead_of_other(statement):
+def semi_colon_instead_of_comma(statement):
     """Writing a semi colon as a typo"""
     _ = current_lang.translate
 
     if statement.bad_token != ";":
         return {}
 
-    new_statement = fixers.replace_token(
-        statement.statement_tokens, statement.bad_token, ":"
-    )
-    if fixers.check_statement(new_statement):
-        cause = _("You wrote a semi-colon, `;`, where a colon, `:`, was expected.\n")
-        hint = _("Did you mean to write a colon `:`?\n")
-        return {"cause": cause, "suggest": hint}
-
     new_statement = fixers.replace_token(
         statement.statement_tokens, statement.bad_token, ","
     )
@@ -705,7 +696,27 @@ def semi_colon_instead_of_other(statement):
         hint = _("Did you mean to write a comma?\n")
         return {"cause": cause, "suggest": hint}
 
-    return {}
+    # perhaps used multiple semi-colons instead of comma
+    if statement.last_token == ";":
+        new_statement = fixers.replace_token(
+            statement.statement_tokens, statement.last_token, ""
+        )
+    else:
+        new_statement = statement.statement
+    while True:
+        tokens = token_utils.tokenize(new_statement)
+        for tok in tokens:
+            if tok == ";":
+                break
+        else:
+            break
+        new_statement = fixers.replace_token(tokens, tok, ",")
+    if fixers.check_statement(new_statement):
+        cause = _("You wrote semi-colons, `;`, where commas were expected.\n")
+        hint = _("Did you mean to write commas?\n")
+        return {"cause": cause, "suggest": hint}
+
+    return {}  # pragma: no cover
 
 
 @add_statement_analyzer
@@ -713,8 +724,12 @@ def invalid_hexadecimal(statement):
     """Identifies problem caused by invalid character in an hexadecimal number."""
     _ = current_lang.translate
 
-    prev = statement.prev_token
-    wrong = statement.bad_token
+    if statement.highlighted_tokens:  # Python 3.10
+        prev = statement.bad_token
+        wrong = statement.next_token
+    else:
+        prev = statement.prev_token
+        wrong = statement.bad_token
     if not (prev.immediately_before(wrong) and prev.string.lower().startswith("0x")):
         return {}
 
@@ -759,13 +774,22 @@ def invalid_name(statement):
 
     first = statement.prev_token
     second = statement.bad_token
+    # New in Python 3.10
+    if (
+        statement.highlighted_tokens is not None
+        and len(statement.highlighted_tokens) > 1
+    ):
+        first = statement.highlighted_tokens[0]
+        second = statement.highlighted_tokens[1]
 
     if not (first.is_number() and second.is_name() and first.end == second.start):
         return {}
 
     cause = _("Valid names cannot begin with a number.\n")
     if first == statement.first_token:  # statement begins with this invalid identifier
-        return {"cause": cause, "suggest": cause}
+        for token in statement.tokens:
+            if token == "=":  # Trying to assign a value to an invalid name
+                return {"cause": cause, "suggest": cause}
 
     if second == "i" and not first.is_complex():
         hint = _("Did you mean `{number}j`?\n").format(number=first)
@@ -779,7 +803,10 @@ def invalid_name(statement):
         return {"cause": cause, "suggest": hint}
 
     if first.is_complex():
-        note = _("[Note: `{first}` is a complex number.]\n").format(first=first)
+        note = _(
+            "[Note: `{first} * {second}` would also be valid\n"
+            "since `{first}` is a complex number.]\n"
+        ).format(first=first, second=second)
         second.string = first.string[-1] + second.string
         first.string = first.string[:-1]
     else:
@@ -814,11 +841,11 @@ def debug_fstring(statement):
         )
         return {"cause": cause}
 
-    return {}
+    return {}  # pragma: no cover
 
 
 @add_statement_analyzer
-def general_fstring_problem(statement=None):
+def general_fstring_problem(statement):  # pragma: no cover
     # General f-string problems are outside of our main priorities.
     _ = current_lang.translate
     if not statement.fstring_error:
@@ -831,22 +858,6 @@ def general_fstring_problem(statement=None):
     return {"cause": cause}
 
 
-@add_statement_analyzer
-def malformed_class_begin_code_block(statement):
-    # Thinking of class simply beginning a code block; something like
-    # class : ...
-    _ = current_lang.translate
-
-    if statement.first_token != "class" or statement.bad_token != ":":
-        return {}
-
-    if statement.prev_token != statement.first_token:
-        return {}
-
-    cause = _("You tried to define a class and did not use the correct syntax.\n")
-    return {"cause": cause}
-
-
 @add_statement_analyzer
 def assign_to_a_keyword(statement):
     """Checks to see if line is of the form 'keyword = ...'"""
@@ -889,9 +900,7 @@ def lambda_with_paren(statement):
         if tok.is_identifier() or tok == ",":
             continue
 
-        return {}
-    else:
-        return {}
+        return {}  # pragma: no cover
 
     cause = _(
         "You cannot have explicit tuples as arguments.\n"
@@ -905,9 +914,16 @@ def lambda_with_paren(statement):
 def wrong_type_declaration(statement):
     _ = current_lang.translate
 
-    if not statement.bad_token.is_identifier():
+    if statement.highlighted_tokens:  # Python 3.10
+        bad_token = statement.next_token
+        prev_token = statement.bad_token
+    else:
+        bad_token = statement.bad_token
+        prev_token = statement.prev_token
+
+    if not bad_token.is_identifier():
         return {}
-    if statement.prev_token.string not in (
+    if prev_token.string not in (
         "int",
         "float",
         "double",
@@ -919,9 +935,7 @@ def wrong_type_declaration(statement):
     ):
         return {}
 
-    new_statement = fixers.replace_token(
-        statement.statement_tokens, statement.prev_token, ""
-    )
+    new_statement = fixers.replace_token(statement.statement_tokens, prev_token, "")
     if fixers.check_statement(new_statement):
         additional = _(
             "If you remove `{type_decl}`, you will have a valid Python statement.\n"
@@ -936,10 +950,8 @@ def wrong_type_declaration(statement):
     cause = _(
         "It looks like you were trying to declare that `{var}` was\n"
         "a variable using the word `{type_decl}`.\n"
-    ).format(
-        var=statement.bad_token, type_decl=statement.prev_token
-    ) + additional.format(
-        type_decl=statement.prev_token
+    ).format(var=statement.bad_token, type_decl=prev_token) + additional.format(
+        type_decl=prev_token
     )
     return {"cause": cause, "suggest": hint}
 
@@ -989,8 +1001,14 @@ def missing_in_with_for(statement):
     """
     _ = current_lang.translate
 
+    index = statement.bad_token_index
+    bad_token = statement.bad_token
+    if statement.highlighted_tokens:  # Python 3.10 may highlight two tokens
+        index += 1
+        bad_token = statement.next_token
+
     nb_for = nb_in = 0
-    for tok in statement.tokens[: statement.bad_token_index]:
+    for tok in statement.tokens[:index]:
         if tok == "for":
             nb_for += 1
         elif tok == "in":
@@ -999,9 +1017,7 @@ def missing_in_with_for(statement):
         return {}
 
     new_statement = fixers.replace_token(
-        statement.statement_tokens,
-        statement.bad_token,
-        "in " + statement.bad_token.string,
+        statement.statement_tokens, bad_token, "in " + bad_token.string
     )
     if fixers.check_statement(new_statement):
         hint = _("Did you forget to write `in`?\n")
@@ -1041,34 +1057,17 @@ def missing_parens_for_range(statement):
 
 def _perhaps_misspelled_keyword(tokens, wrong):
     kwlist = list(keyword.kwlist)
-    # Make sure we try possible typos first
+    if wrong in kwlist:
+        kwlist.remove(wrong)
     similar = utils.get_similar_words(wrong.string, kwlist)
+    if not similar:
+        return []
 
-    # The 'except' and 'with' keywords can apparently appear in similar situations
-    # as 'for', 'if', etc., without raising any SyntaxErrors.
-    # Furthermore, while it might be syntactically correct to have something like
-    # else: statement_on_one_Line
-    # it should be avoided as it could lead to incorrect suggestions.
-    # Also, since 'if' can be used wherever 'elif' is acceptable, we exclude elif
-    excluded = [
-        word
-        for word in ["except", "with", "else", "finally", "try", "elif"]
-        if word not in similar
-    ]
-
-    # 'not' can be used in many places, most of which would likely not make sense
-    # as additional suggestions
-    kwlist.remove("not")
-    words = [word for word in kwlist if word not in similar]
-    similar.extend(words)
     results = []
     for word in similar:
         new_statement = fixers.replace_token(tokens, wrong, word)
         if fixers.check_statement(new_statement):
             results.append((word, new_statement))
-
-    if len(results) > 1:
-        results = [(word, line) for word, line in results if word not in excluded]
     return results
 
 
@@ -1079,26 +1078,89 @@ def misspelled_python_keyword(tokens, bad_token):
     if not results:
         return {}
 
-    if len(results) == 1:
-        word, line = results[0]
-        hint = _("Did you mean `{line}`?\n").format(line=line)
+    word, line = results[0]
+    hint = _("Did you mean `{line}`?\n").format(line=line)
+    cause = _(
+        "Perhaps you meant to write `{keyword}` and made a typo.\n"
+        "The correct line might be `{line}`\n"
+    ).format(keyword=word, line=line)
 
-        cause = _(
-            "Perhaps you meant to write `{keyword}` and made a typo.\n"
-            "The correct line would then be `{line}`\n"
-        ).format(keyword=word, line=line)
+    return {"cause": cause, "suggest": hint}
+
+
+@add_statement_analyzer
+def comprehension_condition_or_tuple(statement):
+    _ = current_lang.translate
+    if not statement.begin_brackets:
+        return {}
+
+    cause_condition = _(
+        "I am guessing that you were writing a comprehension or a generator expression\n"
+        "and use the wrong order for a condition.\n"
+        "The correct order depends if there is an `else` clause or not.\n"
+        "For example, the correct order for a list comprehensions with\n"
+        "condition can be either\n\n"
+        "    [f(x) if condition else other for x in sequence]  # 'if' before 'for'\n\n"
+        "or, if there is no `else`\n\n"
+        "    [f(x) for x in sequence if condition]  # 'if' after 'for'\n\n"
+    )
+
+    cause_tuple = _(
+        "I am guessing that you were writing a comprehension or a generator expression\n"
+        "and forgot to include parentheses around tuples.\n"
+        "As an example, instead of writing\n\n"
+        "    [i, i**2 for i in range(10)]\n\n"
+        "you would need to write\n\n"
+        "    [(i, i**2) for i in range(10)]\n\n"
+    )
+
+    if statement.bad_token == "else":
+        for tok in statement.tokens[0 : statement.bad_token_index]:
+            if tok == "for":
+                cause = cause_condition
+                break
+        else:
+            return {}
+    elif statement.bad_token == "for":
+        for tok in statement.tokens[0 : statement.bad_token_index]:
+            if tok == "if":
+                cause = cause_condition
+                break
+        else:
+            found_bracket = False
+            for tok in statement.tokens[0 : statement.bad_token_index]:
+                if tok.string in "([{":
+                    found_bracket = True
+                if tok == "," and found_bracket:
+                    cause = cause_tuple
+                    hint = _("Did you forget parentheses?\n")
+                    return {"cause": cause, "suggest": hint}
+            else:  # skipcq: PYL-W0120
+                return {}
     else:
-        lines = [line for _word, line in results]
-        hint = _("Did you mean `{line}`?\n").format(line=lines[0])
+        return {}
 
-        cause = _(
-            "Perhaps you wrote another word instead of a Python keyword.\n"
-            "If that is the case, perhaps you meant to write one of\n"
-            "the following lines of code which might not raise a `SyntaxError`:\n\n"
-        )
-        for line in lines:
-            cause += f"    {line}\n"
+    return {"cause": cause}
 
+
+@add_statement_analyzer
+def parens_around_exceptions(statement):
+    # keep in sync with message_analyzer.parens_around_exceptions
+    _ = current_lang.translate
+
+    if statement.bad_token != "," or statement.first_token != "except":
+        return {}
+
+    for tok in statement.tokens[1 : statement.bad_token_index]:
+        if not tok.is_identifier() and tok != ",":
+            return {}
+
+    hint = _("Did you forget parentheses?\n")
+    cause = _(
+        "I am guessing that you wanted to use an `except` statement\n"
+        "with multiple exception types. If that is the case, you must\n"
+        "surround them with parentheses.\n"
+    )
     return {"cause": cause, "suggest": hint}
 
 
@@ -1106,7 +1168,7 @@ def misspelled_python_keyword(tokens, bad_token):
 def current_is_misspelled_python_keyword(statement):
     _ = current_lang.translate
 
-    if not statement.bad_token.is_identifier():
+    if not statement.bad_token.is_name():
         return {}
     return misspelled_python_keyword(statement.tokens, statement.bad_token)
 
@@ -1115,7 +1177,7 @@ def current_is_misspelled_python_keyword(statement):
 def previous_is_misspelled_python_keyword(statement):
     _ = current_lang.translate
 
-    if not statement.prev_token.is_identifier():
+    if not statement.prev_token.is_name():
         return {}
     return misspelled_python_keyword(statement.tokens, statement.prev_token)
 
@@ -1126,16 +1188,23 @@ def space_in_variable_name(statement):
     # my name = André
     _ = current_lang.translate
 
+    bad_token = statement.bad_token
+    prev_token = statement.prev_token
+    if statement.highlighted_tokens:  # Python 3.10
+        bad_token = statement.next_token
+        prev_token = statement.bad_token
+
     if not (
-        statement.bad_token.is_identifier()
-        and statement.prev_token.is_identifier
-        and statement.prev_token is statement.first_token
+        bad_token.is_identifier()
+        and prev_token.is_identifier
+        and prev_token is statement.first_token
     ):
         return {}
 
     first_tokens = []
     for tok in statement.tokens:
-        if str(tok) in ("=", ":="):
+        if tok == "=":
+            # Note: there could be other errors
             cause = _("You cannot have spaces in identifiers (variable names).\n")
             hint = _("Did you mean `{name}`?\n").format(name="_".join(first_tokens))
             return {"cause": cause, "suggest": hint}
@@ -1166,17 +1235,16 @@ def impossible_binary_fstring(statement):
 
 def _add_comma_or_operator(tokens, tok, comma_first=True):
     if comma_first:
-        operators = ",", " +", " -", " *", " in"
+        operators = ",", " +", " -", " *", " in "
     else:
-        operators = " +", " -", " *", ",", " in"
+        operators = " +", " -", " *", ",", " in "
     results = []
     for operator in operators:
-        if operator == " in" and results:
+        if operator == " in " and results:
             break
         new_statement = fixers.modify_token(tokens, tok, append=operator)
         if fixers.check_statement(new_statement):
-            # if operator == "," and results:  # So that it prints correctly
-            #     operator = ','
+
             results.append((operator.strip(), new_statement))
     return results
 
@@ -1214,6 +1282,9 @@ def missing_comma_or_operator(statement):
 
     bad_token = statement.bad_token
     prev_token = statement.prev_token
+    if statement.highlighted_tokens:  # Python 3.10 may highlight two tokens
+        bad_token = statement.next_token
+        prev_token = statement.bad_token
     if not (
         bad_token.is_identifier() or bad_token.is_number() or bad_token.is_string()
     ) and not (
@@ -1242,13 +1313,7 @@ def missing_comma_or_operator(statement):
     if not results:
         return {}
 
-    if (
-        len(results) == 1
-        # Note: "async def" statements should have been transformed
-        # into "def" statements prior to reaching this stage.
-        or statement.first_token == "def"
-        or statement.first_token == "class"
-    ):
+    if len(results) == 1 or statement.first_token.string in ["def", "async", "class"]:
         operator, line = results[0]
         # reducing multiple spaces to single space for nicer display
         temp = line.split(" ")
@@ -1372,98 +1437,12 @@ def from_import_as(statement):
     return {"cause": cause}
 
 
-@add_statement_analyzer
-def comprehension_condition_or_tuple(statement):
-    _ = current_lang.translate
-    if not statement.begin_brackets:
-        return {}
-
-    cause_condition = _(
-        "I am guessing that you were writing a comprehension or a generator expression\n"
-        "and use the wrong order for a condition.\n"
-        "The correct order depends if there is an `else` clause or not.\n"
-        "For example, the correct order for a list comprehensions with\n"
-        "condition can be either\n\n"
-        "    [f(x) if condition else other for x in sequence]  # 'if' before 'for'\n\n"
-        "or, if there is no `else`\n\n"
-        "    [f(x) for x in sequence if condition]  # 'if' after 'for'\n\n"
-    )
-
-    cause_tuple = _(
-        "I am guessing that you were writing a comprehension or a generator expression\n"
-        "and forgot to include parentheses around tuples.\n"
-        "As an example, instead of writing\n\n"
-        "    [i, i**2 for i in range(10)]\n\n"
-        "you would need to write\n\n"
-        "    [(i, i**2) for i in range(10)]\n\n"
-    )
-
-    if statement.bad_token == "else":
-        for tok in statement.tokens[0 : statement.bad_token_index]:
-            if tok == "for":
-                cause = cause_condition
-                break
-        else:
-            return {}
-    elif statement.bad_token == "for":
-        for tok in statement.tokens[0 : statement.bad_token_index]:
-            if tok == "if":
-                cause = cause_condition
-                break
-        else:
-            found_bracket = False
-            for tok in statement.tokens[0 : statement.bad_token_index]:
-                if tok.string in "([{":
-                    found_bracket = True
-                if tok == "," and found_bracket:
-                    cause = cause_tuple
-                    hint = _("Did you forget parentheses?\n")
-                    return {"cause": cause, "suggest": hint}
-            else:  # skipcq: PYL-W0120
-                return {}
-    else:
-        return {}
-
-    return {"cause": cause}
-
-
-@add_statement_analyzer
-def parens_around_exceptions(statement):
-    # keep in sync with message_analyzer.parens_around_exceptions
-    _ = current_lang.translate
-
-    if statement.bad_token != "," or statement.first_token != "except":
-        return {}
-
-    for tok in statement.tokens[1 : statement.bad_token_index]:
-        if not tok.is_identifier() and tok != ",":
-            return {}
-
-    hint = _("Did you forget parentheses?\n")
-    cause = _(
-        "I am guessing that you wanted to use an `except` statement\n"
-        "with multiple exception types. If that is the case, you must\n"
-        "surround them with parentheses.\n"
-    )
-    python_link = _(
-        "https://docs.python.org/3/tutorial/errors.html#handling-exceptions"
-    )
-    return {
-        "cause": cause + "\n" + use_www(),
-        "suggest": hint,
-        "python_link": python_link,
-    }
-
-
 @add_statement_analyzer
 def duplicate_token(statement):
     _ = current_lang.translate
     if statement.bad_token != statement.prev_token:
         return {}
-    if len(statement.bad_token.string) == 1:
-        bad_token = "[`{bad_token}`]".format(bad_token=statement.bad_token)
-    else:
-        bad_token = "`{bad_token}`".format(bad_token=statement.bad_token)
+    bad_token = "`{bad_token}`".format(bad_token=statement.bad_token)
     cause = _(
         "I am guessing that you wrote {bad_token} twice in a row by mistake.\n"
         "If that is the case, you need to remove the second one.\n"
@@ -1488,26 +1467,22 @@ def extra_token(statement):
     )
 
     if fixers.check_statement(new_statement):
-        if len(statement.bad_token.string) == 1:
-            bad_token = "[`{bad_token}`]".format(bad_token=statement.bad_token)
-        else:
-            bad_token = "`{bad_token}`".format(bad_token=statement.bad_token)
+        bad_token = "`{bad_token}`".format(bad_token=statement.bad_token)
         cause = _(
             "I am guessing that you wrote {bad_token} by mistake.\n"
             "Removing it and writing `{line}` seems to fix the error.\n"
         ).format(bad_token=bad_token, line=new_statement)
-        hint = _("Did you write {bad_token} by mistake?\n").format(
-            bad_token=bad_token
-        )
+        hint = _("Did you write {bad_token} by mistake?\n").format(bad_token=bad_token)
         return {"cause": cause, "suggest": hint}
     else:
         return {}
 
+
 # Keep last
 @add_statement_analyzer
 def unclosed_bracket(statement):
     _ = current_lang.translate
-    if not statement.statement_brackets:
+    if not statement.begin_brackets:
         return {}
 
     bracket = statement.begin_brackets[0]
@@ -1525,4 +1500,6 @@ def unclosed_bracket(statement):
         )
         + source
     )
+    if not statement.statement_brackets:
+        cause += _("If this is incorrect, please report this case.\n")
     return {"cause": cause}
